@@ -20,6 +20,15 @@ class AsyncAction:
             params={"qq": self.qq, "timeout": timeout},
         )
 
+    async def close(self):
+        return await self.c.aclose()
+
+    async def __aenter__(self):
+        return self
+
+    async def __aexit__(self, *args):
+        return await self.c.__aexit__(*args)
+
     ############发送相关############
     async def sendFriendText(self, user: int, content: str) -> dict:
         """发送好友文本消息"""
@@ -458,26 +467,27 @@ class AsyncAction:
                 method, httpx.URL(url=path, params=params), json=payload
             )
             resp.raise_for_status()
-        except httpx.HTTPError as e:
-            if isinstance(e, httpx.TimeoutException):
-                logger.warning(f"响应超时，但不代表处理未成功, 结果未知! => {e}")
-            elif isinstance(e, httpx.HTTPStatusError):
-                logger.error(
-                    f"响应码出错 => {resp.status_code}",
-                )
-            else:
-                logger.error(f"请求出错: {traceback.format_exc()}")
+        except httpx.TimeoutException:
+            logger.warning(f"响应超时，但不代表处理未成功, 结果未知!")
+            return {}
+        except httpx.HTTPStatusError:
+            logger.error(
+                f"响应码出错 => {resp.status_code}",  # type:ignore
+            )
+            return {}
+        except Exception:
+            logger.error(f"请求出错: {traceback.format_exc()}")
             return {}
 
         # 处理数据
         try:
             data = resp.json()
-        except Exception as e:
+        except Exception:
             logger.error("API响应结果非json格式")
             return {}
 
         if data is None:
-            logger.error("返回为null")
+            logger.error("返回为null, 该类情况多数是因为响应超时或者该API不存在，或服务端操作超时(此时不代表未成功)")
             return {}
 
         # 返回码提示
