@@ -3,16 +3,20 @@
 """
 
 # NOTE: 这是独立的模块，不应该在框架其他位置被导入，以免循环依赖
+import asyncio
 import base64
+import contextvars
 import inspect
 import os
-import re
 import sys
 import threading
-from functools import wraps
+from asyncio import events
+from functools import partial, wraps
 from pathlib import Path
 from time import monotonic as clock
-from typing import Dict, Union
+from typing import Any, Awaitable, Callable, Dict, Union
+
+import httpx
 
 __all__ = [
     "file_to_base64",
@@ -32,8 +36,8 @@ def file_to_base64(path):
 
 def get_cache_dir(dir_name: str) -> Path:
     """获取缓存目录
-    :param dir_name: the cache directory's name.
-    :return: A Path object of the cache directory
+    :param dir_name: 目录(文件夹)名
+    :return: 返回对应目录的Path对象
     """
     # 确定主缓存目录
     for i in os.listdir(Path(".")):
@@ -187,3 +191,71 @@ class SwitcherManager:
         if key not in self.storage:
             self.storage[key] = Switcher(self.init_enabled)
         return self.storage[key]
+
+
+def download(
+    url: str, dist: Union[str, Path], timeout: int = 20, status: bool = True, **kwargs
+):
+    """下载文件
+    :param url: 文件URL
+    :param dist: 下载保存路径
+    :param timeout: 请求连接超时时间
+    :param status: 是否显示当前进度
+    :param kwargs: 额外传入httpx.stream中的参数，如headers
+    :return: 无返回，下载失败则报相关错误
+    """
+    if isinstance(dist, str):
+        dist = Path(dist)
+    try:
+        with httpx.stream("GET", url, timeout=timeout, **kwargs) as resp:
+            try:
+                total = int(resp.headers["content-length"])
+            except:
+                total = None
+
+            print(f"正在下载: {dist}")
+
+            downloaded = -512
+            with open(dist, "wb") as f:
+                for chunk in resp.iter_bytes(512):
+                    f.write(chunk)
+                    downloaded += 512
+
+                    if status and total:
+                        percent = int(100 * downloaded / total)
+                        print("\r|{:100}|{}%".format("#" * percent, percent), end="")
+                if status and total:
+                    print("\r|{:100}|{}%".format("#" * 100, 100))
+    except Exception as e:
+        if dist.exists():
+            os.remove(dist)
+        raise e
+
+
+def to_async(func: Callable[..., Any]) -> Callable[..., Awaitable[Any]]:
+    """将函数包装为异步函数
+    :param func: 被装饰的同步函数
+    """
+
+    @wraps(func)
+    async def _wrapper(*args: Any, **kwargs: Any) -> Any:
+        loop = events.get_running_loop()
+        ctx = contextvars.copy_context()
+        func_call = partial(ctx.run, func, *args, **kwargs)
+        return await loop.run_in_executor(None, func_call)
+
+    return _wrapper
+
+
+async def async_run(func, *args, **kwargs):
+    """异步执行函数
+    提供的任何 *args 和 **kwargs 会被直接传给 func
+
+    :param func: 目标函数
+    """
+    if asyncio.iscoroutine(func):
+        return await func(*args, **kwargs)
+    loop = events.get_running_loop()
+    ctx = contextvars.copy_context()
+    func_call = partial(ctx.run, func, *args, **kwargs)
+    return await loop.run_in_executor(None, func_call)
